# Copyright (C) 2010 Canonical Ltd.
#
# This file is part of tarmac-lpcommit-plugin.
#
# tarmac-lpcommit-plugin is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Affero Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# tarmac-lpcommit-plugin is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Affero Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with tarmac-lpcommit-plugin.  If not, see <http://www.gnu.org/licenses/>.

"""
A Tarmac plugin for checking commit messages according to the Launchpad commit
message formatting rules.

Messages are checked before merging.  Any proposals whose commit message does
not agree with the format are silently passed over.
"""

__metaclass__ = type

import re
from tarmac.exceptions import TarmacMergeError
from tarmac.hooks import tarmac_hooks
from tarmac.plugins import TarmacPlugin


normal_regex = re.compile(r'''
    (?# Strong suggestion: use ``bzr lp-land`` or ``ec2 land``.
    Manual submission notes: your submission message needs [ui=...]
    AND at least one of [r=...] [rs=...] or [release-critical=...]
    AND at least one of [no-qa] [incr] [rollback=...] [bug=...].)
    (?is)
    (?=(\s*\[[^\]]+\])*\s*\[(release-critical=[^\]]+|rs?=[^\]]+)\])
    (?=(\s*\[[^\]]+\])*\s*\[ui=[^\]]+\])
    (?=(\s*\[[^\]]+\])*\s*\[(incr|no-qa|bugs?=\d+(,\s*\d+)*|rollback=\d+)\])
    ''', re.X)

release_critical_regex = re.compile(r'''
    (?# RELEASE CRITICAL ONLY.
    Strong suggestion: use ``bzr lp-land`` or ``ec2 land``.
    Manual submission notes: your submission message needs [ui=...]
    AND [release-critical=...] AND at least one of [r=...] or [rs=...]
    AND at least one of [no-qa] [incr] [rollback=...] [bug=...] )
    (?is)
    (?=(\s*\[[^\]]+\])*\s*\[release-critical=[^\]]+\])
    (?=(\s*\[[^\]]+\])*\s*\[rs?=[^\]]+\])
    (?=(\s*\[[^\]]+\])*\s*\[ui=[^\]]+\])
    (?=(\s*\[[^\]]+\])*\s*\[(incr|no-qa|bugs?=\d+(,\s*\d+)*|rollback=\d+)\])
    ''', re.X)


class LPCommitMessageCheck(TarmacPlugin):

    """A Tarmac plugin for checking a merge's commit message formatting."""

    def run(self, command, target, source, proposal):
        """See `tarmac.plugins.TarmacPlugin.run` for details."""
        commit_format = self.commit_format_for_branch(target)
        if commit_format is None:
            # Don't run this plugin.
            return
        else:
            self.check_message_format(proposal.commit_message)

    def commit_format_for_branch(self, target_branch):
        """Return the commit message checking mode from a Branch config."""
        format_flag = target_branch.config.get('lp_commit_format', None)
        if format_flag == 'normal':
            global normal_regex
            return normal_regex
        elif format_flag == 'release-critical':
            global release_critical_regex
            return release_critical_regex
        elif format_flag is None:
            # The user did not give us a commit format key value, so assume
            # they did not want us to run the plugin.  Return None as a flag.
            return None
        else:
            # The user gave us an unknown commit format keyword.
            raise UnknownFormatConfigValue(
                "The commit message format {0} is not known by this plugin. "
                "Please use one of the known format values "
                "('normal' or 'release-critical').".format(repr(format_flag)))

    def check_message(self, message, format_regex):
        """Check the given commit message for correctness."""
        if not format_regex.match(message):
            raise MessageFormatError


class MessageFormatError(TarmacMergeError):
    """The merge proposal's commit message is not formatted correctly."""
    pass


class UnknownFormatConfigValue(Exception):
    """The user specified an unknown config value for 'lp_commit_format'."""
    pass


tarmac_hooks['tarmac_pre_merge'].hook(LPCommitMessageCheck(),
    'Launchpad commit messsage formatting checker.')
