"""
Test the regexes that Tarmac uses to constrain what the acceptable submission
messages are.
"""

import unittest
import lpcommit
from tarmac.tests.mock import Thing

class BaseTestCase(unittest.TestCase):

    # Subclasses probably want to override this.
    lp_commit_format_config = None

    def setUp(self):
        self.plugin = lpcommit.LPCommitMessageCheck()

    def makeBranchWithFormat(self, lp_commit_format=None):
        """Create a fake Branch object with the given lp_commit_format."""
        return Thing(config=dict(lp_commit_format=lp_commit_format))

    def makeProposalWithMessage(self, commit_message):
        """Create a fake MergeProposal with the given commit message."""
        return Thing(commit_message=commit_message)

    def assertAccepts(self, message):
        command = Thing()
        target  = self.makeBranchWithFormat(self.lp_commit_format_config)
        source  = Thing()
        proposal = self.makeProposalWithMessage(message)

        self.plugin.run(command, target, source, proposal)

        #self.failUnless(self.regex.match(message))

    def assertRejects(self, message):
        command = Thing()
        target  = self.makeBranchWithFormat(self.lp_commit_format_config)
        source  = Thing()
        proposal = self.makeProposalWithMessage(message)

        self.assertRaises(
            lpcommit.MessageFormatError,
            self.plugin.run, command, target, source, proposal)

        #self.failIf(self.regex.match(message))


class PluginActivationTestCase(BaseTestCase):

    # Pretend none of the plugin configuration keywords have been specified in
    # the branch configuration.
    lp_commit_format_config = None

    def testEmptyBranchConfigHasNoFormat(self):
        branch_with_no_format_config = self.makeBranchWithFormat(None)
        self.assertEqual(None, self.plugin.commit_format_for_branch(
            branch_with_no_format_config))

    def testPluginFindsNormalRegexConfigKeyword(self):
        branch = self.makeBranchWithFormat('normal')
        self.assertNotEqual(None, self.plugin.commit_format_for_branch(
            branch))

    def testPluginFindsRCRegexConfigKeyword(self):
        branch = self.makeBranchWithFormat('release-critical')
        self.assertNotEqual(None, self.plugin.commit_format_for_branch(
            branch))

    def testPluginRaisesErrorsOnUnknownFormatKeywords(self):
        branch = self.makeBranchWithFormat('someunknownformat')
        self.assertRaises(
            lpcommit.UnknownFormatConfigValue,
            self.plugin.commit_format_for_branch, branch)

    def testBranchWithNoFormatConfigMatchesAnything(self):
        # Use a known-bad commit message
        self.assertAccepts('thisisformattedbadly[r=foo][ui=none][no-qa] blah')
        self.assertAccepts('this is missing all of the required tags')


class DevelTestCase(BaseTestCase):


    #def testRegexIsHelpful(self):
    #    # We test it this way rather than with startswith so that the error
    #    # message is more friendly.
    #    self.assertEqual(self.help_string,
    #                     self.regex_string[:len(self.help_string)])

    def testClassicMatchWithBug(self):
        self.assertAccepts('[r=foo][ui=none][bug=123] blah')
        self.assertAccepts('[rs=foo][ui=none][bug=123] blah')
        self.assertAccepts('[release-critical=foo][ui=none][bug=123] blah')

    def testClassicMatchWithNoQA(self):
        self.assertAccepts('[r=foo][ui=none][no-qa] blah')
        self.assertAccepts('[rs=foo][ui=none][no-qa] blah')
        self.assertAccepts('[release-critical=foo][ui=none][no-qa] blah')

    def testClassicMatchWithIncremental(self):
        self.assertAccepts('[r=foo][ui=none][incr] blah')
        self.assertAccepts('[rs=foo][ui=none][incr] blah')
        self.assertAccepts('[release-critical=foo][ui=none][incr] blah')

    def testClassicMatchWithRollback(self):
        self.assertAccepts('[r=foo][ui=none][rollback=1234] blah')
        self.assertAccepts('[rs=foo][ui=none][rollback=1234] blah')
        self.assertAccepts(
            '[release-critical=foo][ui=none][rollback=1234] blah')

    def testClassicMatchWithAltBugs(self):
        self.assertAccepts('[r=foo][ui=none][bug=123, 456] blah')
        self.assertAccepts('[r=foo][ui=none][bug=123,456] blah')
        self.assertAccepts('[r=foo][ui=none][bugs=123] blah')
        self.assertAccepts('[r=foo][ui=none][bugs=123, 456] blah')
        self.assertAccepts('[r=foo][ui=none][bugs=123,456] blah')

    def testTestfixIsIgnored(self):
        self.assertAccepts('[testfix][r=foo][ui=none][bug=123] blah')

    def testOtherRandomBracketedThingsAreAlsoIgnored(self):
        self.assertAccepts('[hoohaa][r=foo][ui=none][bug=123, 456] blah')
        self.assertAccepts('[testfix][hoohaa][r=foo][ui=none][bug=123,456] blah')
        self.assertAccepts(
            '[testfix][hoohaa][r=foo][pablum][ui=none][bugs=123,456] blah')
        self.assertAccepts(
            '[testfix][r=foo][ui=none][Fflewddur Fflam][bug=123] blah')
        self.assertAccepts(
            '[testfix][rs=foo][ui=none][bug=123][scintillating] blah')

    def testAlternateOrders(self):
        self.assertAccepts('[ui=none][r=foo][bug=123] blah')
        self.assertAccepts('[ui=none][bug=123][rs=foo] blah')
        self.assertAccepts('[ui=none][incr][rs=foo] blah')
        self.assertAccepts('[no-qa][ui=none][rs=foo] blah')
        self.assertAccepts('[incr][ui=none][bug=123][rs=foo] blah')

    def testSpacesOutsideBracketsAreFine(self):
        self.assertAccepts(' [r=foo][ui=none][no-qa] blah')
        self.assertAccepts('[r=foo] [ui=none][no-qa] blah')
        self.assertAccepts('[r=foo][ui=none] [no-qa] blah')
        self.assertAccepts('  [r=foo]   [ui=none]  [no-qa] blah')

    def testBracketedThingsMustBeFirst(self):
        self.assertRejects('whatever[r=foo][ui=none][no-qa] blah')
        self.assertRejects('[r=foo]whatever[ui=none][no-qa] blah')
        self.assertRejects('[r=foo][ui=none]whatever[no-qa] blah')

    def testMissingBitsAreNotAllowed(self):
        self.assertRejects('[ui=none][bug=123, 456] blah')
        self.assertRejects('[r=foo][bug=123,456] blah')
        self.assertRejects('[r=foo][ui=none] blah')


#class DevelReleaseCriticalTestCase(StringTestCase):
#    """During "release-critical" but with devel still open"""
#
#    regex_string = (
#        r"(?# RELEASE CRITICAL ONLY.  "
#        r"Strong suggestion: use ``bzr lp-land`` or ``ec2 land``.  "
#        r"Manual submission notes: your submission message needs [ui=...] "
#        r"AND [release-critical=...] "
#        r"AND at least one of [r=...] or [rs=...] "
#        r"AND at least one of [no-qa] [incr] [rollback=...] [bug=...] )"
#        r"(?is)"
#        r"(?=(\s*\[[^\]]+\])*\s*\[release-critical=[^\]]+\])"
#        r"(?=(\s*\[[^\]]+\])*\s*\[rs?=[^\]]+\])"
#        r"(?=(\s*\[[^\]]+\])*\s*\[ui=[^\]]+\])"
#        r"(?=(\s*\[[^\]]+\])*\s*\[(incr|no-qa|bugs?=\d+(,\s*\d+)*|rollback=\d+)\])"
#        )
#
#    def testClassicMatchWithBug(self):
#        self.assertAccepts(
#            '[release-critical=foo][r=foo][ui=none][bug=123] blah')
#        self.assertAccepts(
#            '[release-critical=foo][rs=foo][ui=none][bug=123] blah')
#
#    def testClassicMatchWithNoQA(self):
#        self.assertAccepts(
#            '[release-critical=foo][r=foo][ui=none][no-qa] blah')
#        self.assertAccepts(
#            '[release-critical=foo][rs=foo][ui=none][no-qa] blah')
#
#    def testClassicMatchWithIncremental(self):
#        self.assertAccepts(
#            '[release-critical=foo][r=foo][ui=none][incr] blah')
#        self.assertAccepts(
#            '[release-critical=foo][rs=foo][ui=none][incr] blah')
#
#    def testClassicMatchWithRollback(self):
#        self.assertAccepts(
#            '[release-critical=foo][r=foo][ui=none][rollback=1234] blah')
#        self.assertAccepts(
#            '[release-critical=foo][rs=foo][ui=none][rollback=1234] blah')
#
#    def testClassicMatchWithAltBugs(self):
#        self.assertAccepts(
#            '[release-critical=foo][r=foo][ui=none][bug=123, 456] blah')
#        self.assertAccepts(
#            '[release-critical=foo][r=foo][ui=none][bug=123,456] blah')
#        self.assertAccepts(
#            '[release-critical=foo][r=foo][ui=none][bugs=123] blah')
#        self.assertAccepts(
#            '[release-critical=foo][r=foo][ui=none][bugs=123, 456] blah')
#        self.assertAccepts(
#            '[release-critical=foo][r=foo][ui=none][bugs=123,456] blah')
#
#    def testTestfixIsIgnored(self):
#        self.assertAccepts(
#            '[testfix][release-critical=foo][r=foo][ui=none][bug=123] blah')
#
#    def testOtherRandomBracketedThingsAreAlsoIgnored(self):
#        self.assertAccepts(
#            '[hoohaa][release-critical=foo][r=foo][ui=none][bug=123, 456] '
#            'blah')
#        self.assertAccepts(
#            '[testfix][hoohaa][release-critical=foo][r=foo][ui=none]'
#            '[bug=123,456] blah')
#        self.assertAccepts(
#            '[testfix][release-critical=foo][hoohaa][r=foo][pablum][ui=none]'
#            '[bugs=123,456] blah')
#        self.assertAccepts(
#            '[testfix][release-critical=foo][r=foo][ui=none][Fflewddur Fflam]'
#            '[bug=123] blah')
#        self.assertAccepts(
#            '[testfix][release-critical=foo][rs=foo][ui=none][bug=123]'
#            '[scintillating] blah')
#
#    def testAlternateOrders(self):
#        self.assertAccepts(
#            '[ui=none][release-critical=foo][r=foo][bug=123, 456] blah')
#        self.assertAccepts(
#            '[ui=none][r=foo][release-critical=foo][bug=123,456] blah')
#        self.assertAccepts(
#            '[ui=none][r=foo][bugs=123,456][release-critical=foo] blah')
#        self.assertAccepts(
#            '[ui=none][release-critical=foo][r=foo][bug=123] blah')
#        self.assertAccepts(
#            '[ui=none][bug=123][release-critical=foo][rs=foo] blah')
#        self.assertAccepts(
#            '[r=foo][ui=none][bug=123][release-critical=foo] blah')
#        self.assertAccepts(
#            '[ui=none][incr][rs=foo][release-critical=foo] blah')
#        self.assertAccepts(
#            '[ui=none][no-qa][rs=foo][release-critical=foo] blah')
#        self.assertAccepts(
#            '[ui=none][incr][bug=123][rs=foo][release-critical=foo] blah')
#
#    def testSpacesOutsideBracketsAreFine(self):
#        self.assertAccepts(
#            ' [release-critical=foo][r=foo][ui=none][no-qa] blah')
#        self.assertAccepts(
#            '[release-critical=foo] [r=foo][ui=none][no-qa] blah')
#        self.assertAccepts(
#            '[release-critical=foo][r=foo] [ui=none][no-qa] blah')
#        self.assertAccepts(
#            '[release-critical=foo][r=foo][ui=none] [no-qa] blah')
#        self.assertAccepts(
#            '  [release-critical=foo]  [r=foo]   [ui=none]  [no-qa] blah')
#
#    def testBracketedThingsMustBeFirst(self):
#        self.assertRejects(
#            'whatever[release-critical=foo][r=foo][ui=none][no-qa] blah')
#        self.assertRejects(
#            '[release-critical=foo]whatever[r=foo][ui=none][no-qa] blah')
#        self.assertRejects(
#            '[release-critical=foo][r=foo]whatever[ui=none][no-qa] blah')
#        self.assertRejects(
#            '[release-critical=foo][r=foo][ui=none]whatever[no-qa] blah')
#
#    def testMissingBitsAreNotAllowed(self):
#        self.assertRejects('[r=foo][ui=none][bug=123, 456] blah')
#        self.assertRejects('[rs=foo][ui=none][bug=123, 456] blah')
#        self.assertRejects('[release-critical=foo][ui=none][bug=123, 456] blah')
#        self.assertRejects('[release-critical=foo][r=foo][bug=123,456] blah')
#        self.assertRejects('[release-critical=foo][r=foo][ui=none] blah')
#
#
#class DbDevelTestCase(DevelTestCase):
#
#    section = '/home/pqm/archives/rocketfuel/launchpad/db-devel'
#    help_string = DevelTestCase.help_string[:-1] + (
#        " Note that [rs=buildbot-poller] also works, without anything else, "
#        "but that's just for the buildbot poller. )")
#
#    def testBuildbotPollerIsAccepted(self):
#        self.assertAccepts('[rs=buildbot-poller] blah')
#
#
#class DbDevelReleaseCriticalTestCase(DevelReleaseCriticalTestCase):
#
#    regex_string = (
#        r"(?# RELEASE CRITICAL ONLY.  "
#        r"Strong suggestion: use ``bzr lp-land`` or ``ec2 land``.  "
#        r"Manual submission notes: your submission message needs [ui=...] "
#        r"AND [release-critical=...] "
#        r"AND at least one of [r=...] or [rs=...] "
#        r"AND at least one of [no-qa] [incr] [rollback=...] [bug=...]. "
#        r"Note that [rs=buildbot-poller] also works, without anything else, "
#        r"but that's just for the buildbot poller. )"
#        r"(?is)"
#        r"\s*("
#          r"(?=(\s*\[[^\]]+\])*\[rs=buildbot-poller\])"
#        r"|"
#          r"(?=(\s*\[[^\]]+\])*\s*\[release-critical=[^\]]+\])"
#          r"(?=(\s*\[[^\]]+\])*\s*\[rs?=[^\]]+\])"
#          r"(?=(\s*\[[^\]]+\])*\s*\[ui=[^\]]+\])"
#          r"(?=(\s*\[[^\]]+\])*\s*\[(incr|no-qa|bugs?=\d+(,\s*\d+)*|rollback=\d+)\])"
#        r")")
#
#    def testBuildbotPollerIsAccepted(self):
#        self.assertAccepts('[rs=buildbot-poller] blah')


if __name__ == '__main__':
    unittest.main()
